package com.ruoyi.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.domain.EmployeeBalance;
import com.ruoyi.service.IEmployeeBalanceService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 员工余额Controller
 * 
 * @author ruoyi
 * @date 2025-04-21
 */
@RestController
@RequestMapping("/system/balance")
public class EmployeeBalanceController extends BaseController
{
    @Autowired
    private IEmployeeBalanceService employeeBalanceService;

    /**
     * 查询员工余额列表
     */
    @PreAuthorize("@ss.hasPermi('system:balance:list')")
    @GetMapping("/list")
    public TableDataInfo list(EmployeeBalance employeeBalance)
    {
        startPage();
        List<EmployeeBalance> list = employeeBalanceService.selectEmployeeBalanceList(employeeBalance);
        return getDataTable(list);
    }

    /**
     * 导出员工年假列表
     */
    @PreAuthorize("@ss.hasPermi('system:balance:export')")
    @Log(title = "员工年假", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EmployeeBalance employeeBalance)
    {
        List<EmployeeBalance> list = employeeBalanceService.selectEmployeeBalanceList(employeeBalance);
        ExcelUtil<EmployeeBalance> util = new ExcelUtil<EmployeeBalance>(EmployeeBalance.class);
        util.exportExcel(response, list, "员工年假数据");
    }

    /**
     * 获取员工余额详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:balance:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(employeeBalanceService.selectEmployeeBalanceById(id));
    }

    /**
     * 新增员工余额
     */
    @PreAuthorize("@ss.hasPermi('system:balance:add')")
    @Log(title = "员工余额", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EmployeeBalance employeeBalance)
    {
        return toAjax(employeeBalanceService.insertEmployeeBalance(employeeBalance));
    }

    /**
     * 修改员工余额
     */
    @PreAuthorize("@ss.hasPermi('system:balance:edit')")
    @Log(title = "员工余额", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EmployeeBalance employeeBalance)
    {
        return toAjax(employeeBalanceService.updateEmployeeBalance(employeeBalance));
    }

    /**
     * 删除员工余额
     */
    @PreAuthorize("@ss.hasPermi('system:balance:remove')")
    @Log(title = "员工余额", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(employeeBalanceService.deleteEmployeeBalanceByIds(ids));
    }
}
