package com.ruoyi.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.domain.FyglDailyReimbursement;
import com.ruoyi.domain.dto.*;
import com.ruoyi.domain.vo.FyglDailyReimbursementVo;
import com.ruoyi.mapper.FyglDailyReimbursementMapper;
import com.ruoyi.service.IFyglDailyReimbursementService;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.utils.SerialNumberGenerator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Map;

/**
 * 日常报销Service业务层处理
 *
 * @author ruoyi
 * @date 2025-02-28
 */
@Service
public class FyglDailyReimbursementServiceImpl implements IFyglDailyReimbursementService
{
    @Autowired
    private FyglDailyReimbursementMapper fyglDailyReimbursementMapper;

    @Autowired
    private SerialNumberGenerator serialNumberGenerator;

    @Autowired
    private SysUserMapper sysUserMapper;

    /**
     * 查询日常报销
     *
     * @param id 日常报销主键
     * @return 日常报销
     */
    @Override
    public FyglDailyReimbursementDetailDTO selectFyglDailyReimbursementById(Long id)
    {
        FyglDailyReimbursementDetailDTO fyglDailyReimbursement = fyglDailyReimbursementMapper.selectFyglDailyReimbursementById(id);

        List<FyglDailyReimbursementDetailitemListDTO> childlist = fyglDailyReimbursementMapper.selectFyglDailyReimbursementItemListById(Long.valueOf(fyglDailyReimbursement.getId()));


        fyglDailyReimbursement.setChildList(childlist);
        return fyglDailyReimbursement;
    }

    /**
     * 查询日常报销列表
     *
     * @return 日常报销
     */
    @Override
    public List<FyglDailyReimbursementDTO> selectFyglDailyReimbursementList(FyglDailyReimbursementVo fvo)
    {
        if (!(fvo.getUserName() == null || fvo.getUserName().isEmpty())) {
            SysUser sysUser = sysUserMapper.selectUserByUserName(fvo.getUserName());
            if (sysUser != null) {
                fvo.setUserId(sysUser.getUserId());
            }else fvo.setUserId(-9999999L);
        }
        System.out.println("fvo = " + fvo);
        List<FyglDailyReimbursementDTO> fyglDailyReimbursements = fyglDailyReimbursementMapper.selectFyglDailyReimbursementList(fvo);
        System.out.println("fvo结果 = " + fyglDailyReimbursements);
        for(FyglDailyReimbursementDTO item:fyglDailyReimbursements){
            FyglDailyReimbursementVo f1 = new FyglDailyReimbursementVo();
            f1.setId(item.getId());
            f1.setStatus(fvo.getStatus());
            f1.setReimbursement(fvo.getReimbursement());
            List<FyglDailyReimbursementTableItemDTO> itemList = fyglDailyReimbursementMapper.getItemList(f1);
            item.setTableItemList(itemList);
        }
        return fyglDailyReimbursements;
    }

    @Override
    public FyglProjectDTO getProjectDetail(Long pid) {
        return fyglDailyReimbursementMapper.getProjectDetail(pid);
    }

    /**
     * 新增日常报销
     *
     * @param fyglDailyReimbursement 日常报销
     * @return 结果
     */
    @Override
    @Transactional
    public int insertFyglDailyReimbursement(FyglDailyReimbursement fyglDailyReimbursement)
    {  fyglDailyReimbursement.setCreateBy(SecurityUtils.getUsername());
        fyglDailyReimbursement.setUpdateBy(SecurityUtils.getUsername());
        fyglDailyReimbursement.setCreateTime(DateUtils.getNowDate());
        fyglDailyReimbursement.setUpdateTime(DateUtils.getNowDate());

        String s = serialNumberGenerator.generateSerialNumber("rcbx");
        fyglDailyReimbursement.setSerialNumber(s);

        //先插入最外层表
//        int i1 = fyglDailyReimbursementMapper.insertFyglDailyReimbursement(fyglDailyReimbursement);
        //第二层表
        fyglDailyReimbursement.setStatus(2);

        fyglDailyReimbursement.setRegistrationTime(DateUtils.getNowDate());
        System.out.println();
        int i2 = fyglDailyReimbursementMapper.insertFyglDailyReimbursement2(fyglDailyReimbursement);
        //插入第三层表
        for(FyglDailyReimbursementDetailitemListDTO item:fyglDailyReimbursement.getChildList()){
            item.setRid(fyglDailyReimbursement.getRid());
            //发起插入请求
            fyglDailyReimbursementMapper.insertFyglDailyReimbursement3(item);
        }
        System.out.println("插入对象fyglDailyReimbursement = " + fyglDailyReimbursement);

        return i2;
    }

    @Override
    public List<FyglProjectList> getProjectList() {
        return fyglDailyReimbursementMapper.getProjectList();
    }

    /**
     * 修改日常报销
     *
     * @param fyglDailyReimbursement 日常报销
     * @return 结果
     */
    @Override
    @Transactional
    public int updateFyglDailyReimbursement(FyglDailyReimbursement fyglDailyReimbursement)
    {
        fyglDailyReimbursement.setRegistrationTime(DateUtils.getNowDate());
        fyglDailyReimbursement.setUpdateBy(SecurityUtils.getUsername());
        fyglDailyReimbursement.setUpdateTime(DateUtils.getNowDate());
        System.out.println("修改日常 = " + fyglDailyReimbursement);

        //fygl_daily_rembursement_relevancy_r 修改
        int i1 = fyglDailyReimbursementMapper.updateFyglDailyReimbursement(fyglDailyReimbursement);
        //再删除所有关联的子表数据
        int i3 = fyglDailyReimbursementMapper.delDataByRid(fyglDailyReimbursement.getId());
        //再循环插入子表数据 fygl_daily_reimbursement_relevawncy
        //插入第三层表
        for(FyglDailyReimbursementDetailitemListDTO item:fyglDailyReimbursement.getChildList()){
            item.setRid(fyglDailyReimbursement.getId());
            //发起插入请求
            fyglDailyReimbursementMapper.insertFyglDailyReimbursement3(item);
        }
        return i3;
    }

    /**
     * 批量删除日常报销
     *
     * @param ids 需要删除的日常报销主键
     * @return 结果
     */
//    @Override
//    public int deleteFyglDailyReimbursementByIds(Long[] ids)
//    {
//        return fyglDailyReimbursementMapper.deleteFyglDailyReimbursementByIds(ids);
//    }

    /**
     * 删除日常报销信息
     *
     * @param id 日常报销主键
     * @return 结果
     */
    @Override
    @Transactional
    public int deleteFyglDailyReimbursementById(Long id)
    {
        int row = fyglDailyReimbursementMapper.deleteFyglDailyReimbursementById2(id);
        fyglDailyReimbursementMapper.deleteFyglDailyReimbursementById(id);
        return row;
    }

    @Override
    public List<FyglDailyReimbursementDTO> getProjectListByUserId(FyglDailyReimbursementVo fyglDailyReimbursementVO) {
        //获取每个人关联的项目
        fyglDailyReimbursementVO.setCurrentUserId(SecurityUtils.getUserId());
        List<FyglDailyReimbursementDTO> projectListByUserId = fyglDailyReimbursementMapper.getProjectListByUserId(fyglDailyReimbursementVO);
        // 将projectPageParams转成map集合
        String projectPageParams = fyglDailyReimbursementVO.getProjectPageParams();
        if (projectPageParams != null){
            Map<String, Object> projectPageMap = JSON.parseObject(projectPageParams);
            for(FyglDailyReimbursementDTO item:projectListByUserId){
                Object obj = projectPageMap.get(item.getProjectId());
                // 将obj转成map
                Map<String, Integer> map = (Map<String, Integer>) obj;
                Integer pageNum = map.get("pageNum");
                Integer pageSize = map.get("pageSize");
                PageHelper.startPage(pageNum,pageSize);
                fyglDailyReimbursementVO.setCardItemProjectId(item.getProjectId());
                List<FyglDailyReimbursementTableItemDTO> itemList = fyglDailyReimbursementMapper.getItemList(fyglDailyReimbursementVO);
                PageInfo<FyglDailyReimbursementTableItemDTO> tbUserPageInfo = new PageInfo<>(itemList);
                item.setTableItemList(tbUserPageInfo.getList());
                item.setPageNum(pageNum);
                item.setPageSize(pageSize);
                item.setTotal(Math.toIntExact(tbUserPageInfo.getTotal()));
            }
        } else {
            // 初始化执行 默认每页3条数据
            PageHelper.startPage(1,3);
            for(FyglDailyReimbursementDTO item:projectListByUserId){
                // 将obj转成map
                fyglDailyReimbursementVO.setCardItemProjectId(item.getProjectId());
                List<FyglDailyReimbursementTableItemDTO> itemList = fyglDailyReimbursementMapper.getItemList(fyglDailyReimbursementVO);
                PageInfo<FyglDailyReimbursementTableItemDTO> tbUserPageInfo = new PageInfo<>(itemList);
                item.setTableItemList(tbUserPageInfo.getList());
                item.setPageNum(1);
                item.setPageSize(3);
                item.setTotal(Math.toIntExact(tbUserPageInfo.getTotal()));
            }
        }
        System.out.println("参数 = " + fyglDailyReimbursementVO);
        return projectListByUserId;
    }
}
